@@ -93,7 +93,7 @@ our %EXPORT_TAGS = (
 # 'all' => \@EXPORT_OK,
 );
 
-our $VERSION = '1.11-1';
+our $VERSION = '1.12-1';
 
 #
 # Set global constants
@@ -227,6 +227,11 @@ sub handleVNCStartupAuto {
   if (defined($options->{'vncStartup'}) && $options->{'vncStartup'} eq '__AUTO__') {
     # vncStartup was not defined by the user configuration in ~/.vnc/vnc.conf.
     $options->{'vncStartup'} = $vncStartupAuto;
+  } elsif (($options->{'src'}{'vncStartup'} // "undef") eq 'cmdline' &&
+	   ($options->{'src'}{'session-orig'} // "undef") ne 'cmdline') {
+    # Clear config file session arguments if -xstartup specified on cmdline.
+    $options->{'src'}{'session-orig'} = 'cmdline';
+    $options->{'session-orig'} = $options->{'session'} = [];
   }
 }
 
@@ -354,24 +359,8 @@ sub getOptionParseTable($$) {
           }
         },
        "specifies the X11 display to be used." ],
-      [63, 'rfbport=i'         => 'rfbport',
-       "provides the TCP port to be used for the RFB protocol." ],
-      [63, 'localhost:b'       => 'localhost',
-       "if enabled, VNC will only accept connections from localhost." ],
-      [63, 'SecurityTypes=s'   => 'SecurityTypes',
-       "specifies a comma list of security types to offer (None, VncAuth, Plain, TLSNone, TLSVnc, TLSPlain, X509None, X509Vnc, X509Plain). On default, offer only VncAuth." ],
-      [63, 'PasswordFile|rfbauth=s' => 'vncPasswdFile',
-       "specifies the password file for security types VncAuth, TLSVnc, and X509Vnc. On default, ~/.vnc/passwd is used." ],
       # Backward compatible command line option
       [36, 'passwd=s'          => 'vncPasswdFile' ],
-      [63, 'PlainUsers=s'      => 'PlainUsers',
-       "specifies the list of authorized users for the security types Plain, TLSPlain, and X509Plain." ],
-      [63, 'PAMService|pam_service=s' => 'PAMService',
-       "specifies the service name for PAM password validation that is used in case of security types Plain, TLSPlain, or X509Plain. On default, vnc is used if present otherwise tigervnc is used." ],
-      [63, 'X509Key=s'         => 'X509Key',
-       "denotes a X509 certificate key file (PEM format). This is used by the security types X509None, X509Vnc, and X509Plain." ],
-      [63, 'X509Cert=s'        => 'X509Cert',
-       "denotes the corresponding X509 certificate (PEM format)." ],
       [36, 'fg'                => 'fg',
        "if enabled, $PROG will stay in the foreground." ],
       [36, 'useold'            => 'useold',
@@ -382,27 +371,28 @@ sub getOptionParseTable($$) {
        "if enabled, no real action is taken only a simulation of what would be done is performed." ],
       [36, 'I-KNOW-THIS-IS-INSECURE' => 'I-KNOW-THIS-IS-INSECURE' ],
       # Parameters for both Xtigervnc and X0tigervnc (case insensitive)
-      [63, 'UseIPv4:b'         => 'UseIPv4' ],
-      [63, 'UseIPv6:b'         => 'UseIPv6' ],
-      [63, 'rfbunixpath=s'     => 'rfbunixpath' ],
-      [63, 'rfbunixmode=s'     => 'rfbunixmode' ],
-      [63, 'ClientWaitTimeMillis|rfbwait=i' => 'rfbwait' ],
-      [63, 'AcceptCutText:b'   => 'AcceptCutText' ],
-      [63, 'MaxCutText=i'      => 'MaxCutText' ],
-      [63, 'SendCutText:b'     => 'SendCutText' ],
+      [63, 'ZlibLevel=i'       => 'ZlibLevel' ],
+      [63, 'ImprovedHextile:b' => 'ImprovedHextile' ],
+      [63, 'PAMService|pam_service=s' => 'PAMService',
+       "specifies the service name for PAM password validation that is used in case of security types Plain, TLSPlain, or X509Plain. On default, vnc is used if present otherwise tigervnc is used." ],
+      [63, 'PlainUsers=s'      => 'PlainUsers',
+       "specifies the list of authorized users for the security types Plain, TLSPlain, and X509Plain." ],
+      [63, 'GnuTLSPriority=s'  => 'GnuTLSPriority' ],
+      [63, 'QueryConnect:b'    => 'QueryConnect' ],
+      [63, 'AcceptSetDesktopSize:b' => 'AcceptSetDesktopSize' ],
       [63, 'AcceptPointerEvents:b' => 'AcceptPointerEvents' ],
       [63, 'AcceptKeyEvents:b' => 'AcceptKeyEvents' ],
-      [63, 'AcceptSetDesktopSize:b' => 'AcceptSetDesktopSize' ],
       [63, 'DisconnectClients:b' => 'DisconnectClients' ],
       [63, 'NeverShared:b'     => sub {
           if (@_ == 2) {
             if ($_[1] eq '' || $_[1] eq '1') {
               &{$override}('shared', 'never');
-            } elsif ($options->{'shared'} eq 'never') {
+            } elsif (defined($options->{'shared'}) &&
+                     $options->{'shared'} eq 'never') {
               &{$override}('shared', undef);
             }
           } elsif (defined $options->{'shared'}) {
-            return $options->{'shared'} eq 'never';
+            return $options->{'shared'} eq 'never' ? 1 : 0;
           } else {
             return undef;
           }
@@ -411,34 +401,98 @@ sub getOptionParseTable($$) {
           if (@_ == 2) {
             if ($_[1] eq '' || $_[1] eq '1') {
               &{$override}('shared', 'always');
-            } elsif ($options->{'shared'} eq 'always') {
+            } elsif (defined($options->{'shared'}) &&
+                     $options->{'shared'} eq 'always') {
               &{$override}('shared', undef);
             }
           } elsif (defined $options->{'shared'}) {
-            return $options->{'shared'} eq 'always';
+            return $options->{'shared'} eq 'always' ? 1 : 0;
           } else {
             return undef;
           }
         } ],
-      [63, 'Password=s'        => 'Password' ],
       [63, 'Protocol3.3:b'     => 'Protocol3.3' ],
       [63, 'FrameRate=i'       => 'FrameRate' ],
-      [63, 'CompareFB=s'       => 'CompareFB' ],
-      [63, 'ZlibLevel=i'       => 'ZlibLevel' ],
-      [63, 'ImprovedHextile:b' => 'ImprovedHextile' ],
-      [63, 'GnuTLSPriority=s'  => 'GnuTLSPriority' ],
-      [63, 'UseBlacklist:b'    => 'UseBlacklist' ],
-      [63, 'BlacklistThreshold=i' => 'BlacklistThreshold' ],
-      [63, 'BlacklistTimeout=i' => 'BlacklistTimeout' ],
-      [63, 'IdleTimeout=i'     => 'IdleTimeout' ],
-      [63, 'MaxDisconnectionTime=i' => 'MaxDisconnectionTime' ],
-      [63, 'MaxConnectionTime=i' => 'MaxConnectionTime' ],
+      [63, 'CompareFB=i'       => 'CompareFB' ],
       [63, 'MaxIdleTime=i'     => 'MaxIdleTime' ],
-      [63, 'QueryConnect:b'    => 'QueryConnect' ],
-      [63, 'QueryConnectTimeout=i' => 'QueryConnectTimeout' ],
-      [63, 'Log=s'             => 'Log' ],
+      [63, 'MaxConnectionTime=i' => 'MaxConnectionTime' ],
+      [63, 'MaxDisconnectionTime=i' => 'MaxDisconnectionTime' ],
+      [63, 'IdleTimeout=i'     => 'IdleTimeout' ],
       [63, 'RemapKeys=s'       => 'RemapKeys' ],
+      [63, 'BlacklistTimeout=i' => 'BlacklistTimeout' ],
+      [63, 'BlacklistThreshold=i' => 'BlacklistThreshold' ],
+      [63, 'UseBlacklist:b'    => 'UseBlacklist' ],
+      [63, 'Log=s'             => 'Log' ],
+      [63, 'UseIPv6:b'         => 'UseIPv6' ],
+      [63, 'UseIPv4:b'         => 'UseIPv4' ],
+      [63, 'QueryConnectTimeout=i' => 'QueryConnectTimeout' ],
       [63, 'RawKeyboard:b'     => 'RawKeyboard' ],
+      [63, 'interface=s'       => sub {
+          if (@_ == 2) {
+            &{$override}('interface', $_[1]);
+            &{$override}('localhost', undef);
+          } else {
+            return $options->{'interface'};
+          }
+        } ],
+      [63, 'localhost:b'       => 'localhost',
+       "if enabled, VNC will only accept connections from localhost." ],
+      [63, 'desktop=s'         => 'desktopName',
+       "specifies the VNC desktop name." ],
+      [63, 'rfbunixmode=s'     => 'rfbunixmode' ],
+      [63, 'rfbunixpath=s'     => 'rfbunixpath' ],
+      [63, 'rfbport=i'         => 'rfbport',
+       "provides the TCP port to be used for the RFB protocol." ],
+      [63, 'X509Key=s'         => 'X509Key',
+       "denotes a X509 certificate key file (PEM format). This is used by the security types X509None, X509Vnc, and X509Plain." ],
+      [63, 'X509Cert=s'        => 'X509Cert',
+       "denotes the corresponding X509 certificate (PEM format)." ],
+      [63, 'Password=s'        => 'Password' ],
+      [63, 'PasswordFile|rfbauth=s' => 'vncPasswdFile',
+       "specifies the password file for security types VncAuth, TLSVnc, and X509Vnc. On default, ~/.vnc/passwd is used." ],
+      [63, 'SecurityTypes=s'   => 'SecurityTypes',
+       "specifies a comma list of security types to offer (None, VncAuth, Plain, TLSNone, TLSVnc, TLSPlain, X509None, X509Vnc, X509Plain). On default, offer only VncAuth." ],
+      # Arguments from Xtigervnc (case sensitive)
+      [14, 'auth=s'            => 'xauthorityFile' ],
+      # -inetd is not handled
+      # Parameters for Xtigervnc (case insensitive)
+      [15, 'SendCutText:b'     => 'SendCutText' ],
+      [15, 'AcceptCutText:b'   => 'AcceptCutText' ],
+      [15, 'MaxCutText=i'      => 'MaxCutText' ],
+      [15, 'SendPrimary:b'     => 'SendPrimary' ],
+      [15, 'SetPrimary:b'      => 'SetPrimary' ],
+      [15, 'AllowOverride=s'   => 'AllowOverride' ],
+      [15, 'AvoidShiftNumLock:b' => 'AvoidShiftNumLock' ],
+      # Parameters for X0tigervnc (case insensitive)
+      [51, 'UseSHM:b'          => 'UseSHM' ],
+      [51, 'HostsFile=s'       => 'HostsFile' ],
+      [51, 'MaxProcessorUsage=i' => 'MaxProcessorUsage' ],
+      [51, 'PollingCycle=i'    => 'PollingCycle' ],
+      [49, 'Geometry=s'        => $storeGeometry = sub {
+          if (@_ == 2) {
+            my $Geometry = $_[1] // "undef";
+            unless ($Geometry =~ /^(\d+)x(\d+)(?:([+-]\d+)([+-]\d+))?$/) {
+              die "Invalid $_[0] $_[1]!";
+            }
+            &{$override}('Geometry', $_[1]);
+          } else {
+            return $options->{'Geometry'};
+          }
+        },
+        "specifies the screen area that will be shown to VNC clients. The format is widthxheight+xoffset+yoffset, where `+' signs can be replaced with `-' signs to specify offsets from the right and/or from the bottom of the screen. Offsets are optional, +0+0 is assumed by default (top left corner). If the argument is  empty, full screen is shown to VNC clients (this is the default)." ],
+      # Config file only stuff
+      [ 2, 'sslAutoGenCertCommand=s' => 'sslAutoGenCertCommand' ],
+      [ 2, 'vncUserDir=s'      => 'vncUserDir' ],
+      [ 2, 'vncStartup=s'      => 'vncStartup' ],
+      [ 2, 'xauthorityFile=s'  => 'xauthorityFile' ],
+      [ 2, 'getDefaultFrom=s'  => 'getDefaultFrom' ],
+      [ 2, 'scrapingGeometry=s'=> $storeGeometry ],
+      # Backward compatible configuration file option for desktop
+      [ 2, 'desktopName=s'     => 'desktopName' ],
+      # Backward compatible configuration file optionfor PasswordFile
+      [ 2, 'vncPasswdFile=s'   => 'vncPasswdFile' ],
+      # Backward compatible configuration file option for fp
+      [ 2, 'fontPath=s'        => 'fontPath' ],
       # Options only for tigervncserver
       [14, 'geometry=s'        => sub {
           if (@_ == 2) {
@@ -518,8 +572,6 @@ sub getOptionParseTable($$) {
           }
         },
        "disables X session startup." ],
-      [15, 'desktop=s'         => 'desktopName',
-       "specifies the VNC desktop name." ],
       # Backward compatible command line option
       [ 4, 'name=s'            => 'desktopName' ],
       [14, 'depth=i'           => sub {
@@ -569,6 +621,12 @@ sub getOptionParseTable($$) {
       [ 6, 'session=s'         => sub {
           if (@_ == 2) {
             my ($sn, $snt);
+
+            # The following options are only defined if the session is given by
+            # one of the /usr/share/xsessions/*.desktop files.
+            delete $options->{'sessionName'};
+            delete $options->{'sessionDesktop'};
+
             if (!defined($_[1]) || $_[1] eq '') {
               $sn = $snt = [];
             } elsif (ref($_[1]) eq 'ARRAY') {
@@ -576,10 +634,12 @@ sub getOptionParseTable($$) {
             } elsif (ref($_[1]) eq '') {
               $sn = $snt = [split(qr{\s+}, $_[1])];
               unless ($options->{'remote'}) {
-                my $sessionCommand = loadXSession($_[1]) unless $_[1] =~ m{/};
+                my ($sc, $sd) = loadXSession($_[1]) unless $_[1] =~ m{/};
                 my $found = 0;
-                if (defined $sessionCommand) {
-                  $snt = [$sessionCommand];
+                if (defined $sc) {
+                  $snt = [$sc];
+                  $options->{'sessionName'}    = $_[1];
+                  $options->{'sessionDesktop'} = $sd;
                   $found = 1;
                 } elsif (@{$sn} > 0) {
                   if ($sn->[0] =~ m{/}) {
@@ -613,51 +673,6 @@ sub getOptionParseTable($$) {
           }
         },
        "specifies the X11 session to start with either a command or a session name." ],
-      # Arguments from Xtigervnc (case sensitive)
-      [14, 'auth=s'            => 'xauthorityFile' ],
-      # -inetd is not handled
-      [14, 'interface=s'       => sub {
-          if (@_ == 2) {
-            &{$override}('interface', $_[1]);
-            &{$override}('localhost', undef);
-          } else {
-            return $options->{'interface'};
-          }
-        } ],
-      # Parameters for Xtigervnc (case insensitive)
-      [15, 'AvoidShiftNumLock:b' => 'AvoidShiftNumLock' ],
-      [15, 'AllowOverride=s'   => 'AllowOverride' ],
-      [15, 'SendPrimary:b'     => 'SendPrimary' ],
-      # Parameters for X0tigervnc (case insensitive)
-      [49, 'Geometry=s'        => $storeGeometry = sub {
-          if (@_ == 2) {
-            my $Geometry = $_[1] // "undef";
-            unless ($Geometry =~ /^(\d+)x(\d+)(?:([+-]\d+)([+-]\d+))?$/) {
-              die "Invalid $_[0] $_[1]!";
-            }
-            &{$override}('Geometry', $_[1]);
-          } else {
-            return $options->{'Geometry'};
-          }
-        },
-        "specifies the screen area that will be shown to VNC clients. The format is widthxheight+xoffset+yoffset, where `+' signs can be replaced with `-' signs to specify offsets from the right and/or from the bottom of the screen. Offsets are optional, +0+0 is assumed by default (top left corner). If the argument is  empty, full screen is shown to VNC clients (this is the default)." ],
-      [49, 'HostsFile=s'       => 'HostsFile' ],
-      [49, 'MaxProcessorUsage=i' => 'MaxProcessorUsage' ],
-      [49, 'PollingCycle=i'    => 'PollingCycle' ],
-      [49, 'UseSHM:b'          => 'UseSHM' ],
-      # Config file stuff
-      [ 2, 'sslAutoGenCertCommand=s' => 'sslAutoGenCertCommand' ],
-      [ 2, 'vncUserDir=s'      => 'vncUserDir' ],
-      [ 2, 'vncStartup=s'      => 'vncStartup' ],
-      [ 2, 'xauthorityFile=s'  => 'xauthorityFile' ],
-      [ 2, 'getDefaultFrom=s'  => 'getDefaultFrom' ],
-      [ 2, 'scrapingGeometry=s'=> $storeGeometry ],
-      # Backward compatible configuration file option for desktop
-      [ 2, 'desktopName=s'     => 'desktopName' ],
-      # Backward compatible configuration file optionfor PasswordFile
-      [ 2, 'vncPasswdFile=s'   => 'vncPasswdFile' ],
-      # Backward compatible configuration file option for fp
-      [ 2, 'fontPath=s'        => 'fontPath' ],
     );
 
   my $optionParseTable = [];
@@ -957,7 +972,12 @@ sub loadXSession {
     return;
   }
 
-  return $session{'Exec'};
+  if (defined $session{'DesktopNames'}) {
+    $session{'DesktopNames'} =~ s/;*$//g;
+    $session{'DesktopNames'} =~ s/;;*/:/g;
+  }
+
+  return ($session{'Exec'}, $session{'DesktopNames'});
 }
 
 sub parseCmdLine {
@@ -1380,7 +1400,6 @@ sub getConfig {
       depth                     => 24,
       pixelformat               => undef,
       getDefaultFrom            => undef,
-      rfbwait                   => 30000,
       rfbport                   => undef,
       localhost                 => undef,
       SecurityTypes             =>
@@ -1457,7 +1476,7 @@ Joachim Falk E<lt>joachim.falk@gmx.deE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2021 by Joachim Falk
+Copyright (C) 2021 and 2022 by Joachim Falk
 
 This is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
